{ Copyright (C) 09/09/2004 00:00, Luis R. Hilario B. <info@luisdigital.com>

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}

unit imprimir;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

Type
  THTMLPrint = Class(TObject)
  private
    FArchivo,
    FField_Name: String;
    FReg_Name: String;
    FTable_Name: String;
    XNodeText, YNodeText,
    XNode, YNode: Integer;
    X, Y: Integer;
    FNodeType,
    FNodeName,
    FNodeValue,
    FText: String;
    FStringList: TStrings;
    procedure IniIndex;
    procedure IncIndex(const LengthCol: Integer);
    procedure Node;
    procedure NodeText;
    procedure FillField(const Text: String);
  protected
    property Datos: TStrings Read FStringList Write FStringList;
    function ReplaceText(const NodeType, NodeName,
                               NodeValue, Text: String): Boolean;
  public
    constructor Create(Const ArchivoNombre: String);
    procedure AddRow(Count: Integer);
    procedure Read;
    procedure Write;
    procedure Print(const Text, FieldName: String);
    procedure TablePrint(const Text, FieldName: String; Index: Integer);
    property Table_Name: String Read FTable_Name Write FTable_Name;
    property Reg_Name: String Read FReg_Name Write FReg_Name;
    property Field_Name: String Read FField_Name Write FField_Name;
    property Archivo: String Read FArchivo Write FArchivo;
    destructor Destroy; override;
end;

Const
  Version= 'klinton01_';
  TABLE= 'TABLE';
  TBODY= 'TBODY';
  TR= 'TR';
  TR_END= '/'+TR;
  TBODY_END= '/'+TBODY;
  TABLE_END= '/'+TABLE;
  BODY_END= '/BODY';
  HTML_END= '/HTML';
  HtmlSpace= '&nbsp;';

Var
  HTMLPrint: THTMLPrint;

implementation

{ THTMLPrint }

constructor THTMLPrint.Create(Const ArchivoNombre: String);
begin
  FArchivo := ArchivoNombre;
  FStringList := TStringList.Create;
end;

procedure THTMLPrint.IniIndex;
begin
  X := 1; Y := 0;
end;

procedure THTMLPrint.IncIndex(const LengthCol: Integer);
begin
  if X = LengthCol then
    begin
      inc(Y);
      X := 1;
      while Datos[Y] = '' do inc(Y);
    end else inc(X);
end;

procedure THTMLPrint.Node;
var
  NodeValueExit: Boolean;

begin
  FNodeType := '';
  FNodeName := '';
  FNodeValue := '';
  if Datos[Y][X] = '<' then
  begin
    IncIndex(Length(Datos[Y]));
    while (Datos[Y][X] <> ' ') and (Datos[Y][X] <> '>') do
      begin
        FNodeType := FNodeType + UpCase(Datos[Y][X]);
        IncIndex(Length(Datos[Y]));
      end; //a FNodeType
    //Writeln(FNodeType, '*');

    repeat
    FNodeName := '';
    while Datos[Y][X] = ' ' do IncIndex(Length(Datos[Y]));
    while (Datos[Y][X] <> '=') and (Datos[Y][X] <> '>') do
      begin
        FNodeName := FNodeName + UpCase(Datos[Y][X]);
        IncIndex(Length(Datos[Y]));
      end; //name FNodeName
        //Writeln(' -',FNodeName, '*');

    if Datos[Y][X] = '=' then
      begin
        FNodeValue := '';
        IncIndex(Length(Datos[Y]));
        NodeValueExit := False;
        repeat
          FNodeValue := FNodeValue + Datos[Y][X];
          if Length(FNodeValue) > 1 then
            begin
              if FNodeValue[1] = '"' then
                begin
                  if Datos[Y][X] = '"' then NodeValueExit := True;
                end else
                if Datos[Y][X] = ' ' then NodeValueExit := True;
            end;
          IncIndex(Length(Datos[Y]));
        until NodeValueExit; //Value FNodeValue
        //Writeln('  -',FNodeValue, '*');
        FNodeValue := AnsiExtractQuotedStr(Pchar(FNodeValue), '"');
        FNodeValue := Trim(FNodeValue);
      end; // =

      until Datos[Y][X] = '>'; // ' '
      XNode := X;
      YNode := Y;
      if FNodeType <> HTML_END then IncIndex(Length(Datos[Y]));
  end;
  NodeText;
end;

procedure THTMLPrint.NodeText;
var
  Temp_Y: Integer;

begin
  FText := '';
  if FNodeType <> BODY_END then
  begin
    XNodeText := X;
    YNodeText := Y;
    Temp_Y := Y;

    while Datos[Y][X] = ' ' do IncIndex(Length(Datos[Y]));

    if Datos[Y][X] <> '<' then
      begin
        while Datos[Y][X] <> '<' do
          begin
            if Temp_Y < Y then
              begin
                FText := FText + ' ';
                Temp_Y := Y;
              end;
            FText := FText + Datos[Y][X];
            IncIndex(Length(Datos[Y]));
          end; //Value FText
        //Writeln(FText, '*');

        while Pos(HtmlSpace, FText) > 0 do // Convert HtmlSpace to ' '
          begin
            Insert(' ', FText, Pos(HtmlSpace, FText));
            Delete(FText, Pos(HtmlSpace, FText), Length(HtmlSpace));
          end;
      end;
  end;
end;

procedure THTMLPrint.FillField(const Text: String);
var
  S: String;

begin
  S := Text;
  if Length(Text) > Length(FText) then S := Copy(Text, 1, Length(FText));
  Delete(FText, 1, Length(S));
  FText := TrimRight(S + FText);

  while Pos('  ', FText) > 0 do // Convert ' ' to HtmlSpace
    begin
      Insert(HtmlSpace, FText, Pos('  ', FText)+2);
      Delete(FText, Pos('  ', FText), 1);
    end;

  X := XNodeText;
  Y := YNodeText;
  S := Datos[Y];
  while S[X] <> '<' do
    begin
      Delete(S, X, 1);
      Datos[Y] := S;
      if X > Length(S) then
        begin
          if Datos[Y] = '' then Datos.Delete(Y) else inc(Y);
          X := 1;
          S := Datos[Y];
        end;
    end;

  X := XNodeText;
  Y := YNodeText;
  if X >= Length(Datos[Y]) then
    begin
      Datos[Y] := Datos[Y] + Datos[Y+1];
      Datos.Delete(Y+1);
    end;
  S := Datos[Y];
  Insert(FText, S, x);
  Datos[Y] := S;
end;

function THTMLPrint.ReplaceText(const NodeType, NodeName, NodeValue,
  Text: String): Boolean;
begin
  IniIndex;
  Result := False;
  repeat
    Node;

    if FNodeType <> '' then
      if (FNodeType + FNodeName + FNodeValue = NodeType + NodeName + NodeValue)
        and
         (FText <> '') then
          begin
           FillField(Text);
           Result := True;
          end;

  until (FNodeType = Body_End) or Result;
end;

procedure THTMLPrint.AddRow(Count: Integer);

procedure CopyTbody(const Count: Integer; var BodyNivel: Integer);
var
  S: String;
  X0, Y0,
  X1, Y1,
  Temp_X0,
  Temp_Y0,
  Temp_X1,
  Temp_Y1,
  CopyCount: Integer;

begin
  S := '';
  Temp_X0 := XNode;
  Temp_Y0 := YNode;
  repeat
  Temp_X1 := XNode;
  Temp_Y1 := YNode;
  Node;
  if FNodeType = TBODY then inc(BodyNivel) else
    if FnodeType = TBODY_END then dec(BodyNivel);
  until BodyNivel = 0;

  for CopyCount := 1 to Count do
    begin
      X0 := Temp_X0;
      X1 := Temp_X1;
      Y0 := Temp_Y0;
      Y1 := Temp_Y1;
      repeat
        if X0 = Length(Datos[Y0]) then
          begin
            inc(Y0);
            inc(Y1);
            Datos.Insert(Y1, '');
            X0 := 1;
            X1 := 1;
          end else
          begin
            inc(X0);
            inc(X1);
          end;
        S := Datos[Y1];
        Insert(Datos[Y0][X0], S, X1);
        Datos[Y1] := S;
      until (Y0 = Temp_Y1) and (X0 = Temp_X1);
    end;
end;

const
//  NodeType = 'A';
  Nodename = 'NAME';

var
  TableCount,
  TableNivel,
  BodyNivel: Integer;
  NodeValue: String;

begin
  if Count < 1 then exit;
  TableCount := 0;
  TableNivel := 0;
  BodyNivel := 0;
  NodeValue := Version + Table_Name;
  IniIndex;
  repeat
    Node;

    if FNodeType = TABLE then
      begin
        inc(TableCount);
        if (FNodeName = NodeName) and
           (FNodeValue = NodeValue) then TableNivel := TableCount;
      end else
      if FNodeType = TABLE_END then
        begin
          dec(TableCount);
          if TableCount < TableNivel then TableNivel := 0;
        end;

    if TableNivel > 0 then
      begin
        if FNodeType = TBODY then inc(BodyNivel) else
          if FnodeType = TBODY_END then dec(BodyNivel);

        if BodyNivel > 0 then
          begin
            CopyTbody(Count, BodyNivel);
            FNodeType := Body_End;
          end;
      end;

  until FNodeType = Body_End;
end;

procedure THTMLPrint.Read;
begin
  FStringList.LoadFromFile(FArchivo);
end;

procedure THTMLPrint.Write;
begin
  Datos.SaveToFile(Archivo);
end;

procedure THTMLPrint.Print(const Text, FieldName: String);
const
  NodeType = 'A';
  Nodename = 'NAME';

Var
  NodeValue: String;

begin
  Field_Name := FieldName;
  NodeValue := Version + Table_Name + '_' + Field_Name;
  ReplaceText(NodeType, NodeName, NodeValue, Text);
end;

procedure THTMLPrint.TablePrint(const Text, FieldName: String; Index: Integer);
const
  NodeType = 'A';
  Nodename = 'NAME';

var
  TableCount,
  TableNivel,
  BodyNivel,
  TRNivel,
  RegNivel: Integer;
  TableNodeValue,
  NodeValue: String;

begin
  Field_Name := FieldName;
  NodeValue := Version + Table_Name + '_' + Field_Name;
  TableCount := 0;
  TableNivel := 0;
  BodyNivel := 0;
  TRNivel := 0;
  RegNivel := 0;
  TableNodeValue := Version + Table_Name;
  IniIndex;
  repeat
    Node;

    if FNodeType = TABLE then
      begin
        inc(TableCount);
        if (FNodeName = NodeName) and
           (FNodeValue = TableNodeValue) then TableNivel := TableCount;
      end else
      if FNodeType = TABLE_END then
        begin
          dec(TableCount);
          if TableCount < TableNivel then TableNivel := 0;
        end;

    if TableNivel > 0 then
      begin
        if FNodeType = TBODY then inc(BodyNivel) else
          if FnodeType = TBODY_END then dec(BodyNivel);

        if BodyNivel > 0 then
          begin
            if FNodeType = TR then inc(TRNivel) else
              if FnodeType = TR_END then dec(TRNivel);

            if TRNivel > 0 then
              begin
                if (FNodeType + FNodeName + FNodeValue =
                    NodeType + NodeName + NodeValue) then
                  begin
                    inc(RegNivel);
                    if (RegNivel = Index) and
                       (FText <> '') then
                        begin
                         FillField(Text);
                         FNodeType := Body_End;
                        end;
                  end;
              end;

          end;
      end;

  until FNodeType = Body_End;
end;

destructor THTMLPrint.Destroy;
begin
  FStringList.Free;
  inherited Destroy;
end;

end.

